[![first-timers-only](https://img.shields.io/badge/first--timers--only-friendly-blue.svg?style=flat-square)](https://www.firsttimersonly.com/)
![Open Source Love](https://img.shields.io/badge/Open%20Source-%E2%9D%A4-pink.svg)
![Open Source Love](https://img.shields.io/badge/chat%20on-discord-green.svg)
![MadeinMoris](https://img.shields.io/badge/Made%20in-Moris-green.svg)

# 🍯 honeybot py

## 📮 About
HoneyBot is a python-based IRC bot. (**python3.7**) | If you want to just run the bot, go to the [quick start section](https://github.com/pyhoneybot/honeybot#-quickstart)

Feel free to contribute to the project!
#
![alt text](honeybot_real.png "honeybot logo")

## 🕹 Project Motivation
Implementing the project in Java was weird, py's connect was sleek. Thus, the project stack was shifted over to Python.
If you can think of any features, plugins, or functionality you wish to see in the project. Feel free to add it yourself, or create an issue detailing your ideas. We highly recommend you attempt to implement it yourself first and ask for help in our [discord server](https://discord.gg/E6zD4XT) !

Psst. Since I learnt py through this bot, we decided to keep a new-comers friendly policy. Feeling lost? Just ping.

## 📌 Contributing Countries

🇲🇺 🇺🇸 🇨🇦 🇦🇷 🇮🇳 🇬🇧 🇬🇬 🇧🇷 🇸🇽 🇵🇱 🇩🇪

## 📨 Follow the project on CodeTriage for updates!

Get issues delivered in your inbox.

[![Open Source Helpers](https://www.codetriage.com/pyhoneybot/honeybot/badges/users.svg)](https://www.codetriage.com/pyhoneybot/honeybot)

## Discord
https://discord.gg/E6zD4XT

## :thought_balloon: Project Testimonials
[@TannerFry](https://github.com/TannerFry)

> With experience in programming in Python, and implementing an SMTP email plugin for a different system, picking up HoneyBot and following the documentation provided for new-comers made it very simple to implement the same SMTP email plugin to the HoneyBot system. This was my first time contributing to an open-source
project on GitHub and it was an overall great experience. The welcoming of new contributors and documentation on how to contribute and implement plugins is great for people
who have never contributed to a project before, and Abdur-Rahmaan Janhangeer was extremely helpful when answering my questions and helping me along the way.

[@RiceAbove](https://github.com/RiceAbove)

> HoneyBot is my first time collaborating to an open source project and I'm loving it. Before discovering HoneyBot, I was very intimidated on the idea of working with other people and had no idea what an IRC even was. Now I realize how much fun and rewarding it is to work together on a project with dedicated and friendly individuals. The documentation is easy to follow and everyone is super helpful. I highly recommend any new programmer who want to contribute on an open source project to try out HoneyBot. Personally I enjoy working on this project more than my own schoolwork.

[@justinwalker4179](https://github.com/IronPenguin4179)

> HoneyBot is my first open source project and I had never worked with an IRC before. For school I was required to contribute to projects, but it was always so intimidating to me. I had always heard it gets easier once you've gotten over the fear wall, and that's what HoneyBot did for me. Excellent readme and quick feedback allowed me to make my first plugin. Now I've made many contributions, and look forward to any new issues I can get my hands on. Abdur-Rahmaan Janhangeer has been extremely helpful and I owe him and this project a lot for getting me into the open source world.

[@mboekhold](https://github.com/mboekhold)

> HoneyBot is a very friendly and welcoming community. They provided quick feedback and I would defenitely recommend this project to newcomers to give them that first boost on contributing to open source. HoneyBot helps you all the way there to issuing your first PR, step by step. I contributed a comic plugin, and I was thankful for meeting all the friendly and passionate people who are active in this community.

[@Macr0Nerd](https://github.com/Macr0Nerd)

> I came onto this project in October of 2018. At the time, my experience with open source was nil. I wasn't even sure I was good with python; I just happened to be better than everyone in class. Working on HoneyBot really opened my eyes to the power of community and programming, and has sent me on a path I don't wanna turn from. Just seeing how this project has evolved over the past year and how I've grown with it, I wouldn't trade it for the world. My name might not be on any of the recent plugins or doc strings, but I'll never forget that it'll always be tied to this project and a lot of the code. Also, I don't know if they'll check my commit so go look at the C++ Client! It's cool I promise!

## ✂ Current Features
 * 🍬 OOP architecture
 * 🛰️ keyword parameters
 * 🌵 password security with config file [disabled for now]
 * 🔌 now with plugins

## :mountain: GUI clients

GUI clients are used to manage plugins, launch bot as well as specify credentials.

- [CPP client](https://github.com/pyhoneybot/cpp-client) by [@Macr0Nerd](https://github.com/Macr0Nerd)

## ⌚ Current Plugins

- 💎 bitcoin by [@Macr0Nerd](https://github.com/Macr0Nerd) - get price of bitcoin
- ⏲ caesar cipher by [@kylegalloway](https://github.com/kylegalloway) - encode your text
- 🔢 calc by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ) - evaluates maths expressions
- 📐 maths by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ) - sin cos and the like
- 🍃 conv sniff by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ) - set triggers like how many times a word occur for one or more words and send response
- ❄ greet by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ) - demo plugin
- ⛓ installed_modules by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ) - checks dependencies installed
- 🕶 joke by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ), [@colbyjayallen](https://github.com/colbyjayallen) - get random joke
- ❓ self Trivia by [@ajimenezUCLA](https://github.com/ajimenezUCLA) - random trivia
- 💢 username by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ), [@sseryani](https://github.com/sseryani) - username generator
- 📜 quotes by [@German-Corpaz](https://github.com/German-Corpaz) - inspirational quotes
- 📖 dictionary by [@iamnishant14](https://github.com/iamnishant14) - returns meaning of word
- 🔣 password generator by [@iamnishant14](https://github.com/iamnishant14) - the name tells it all
- 🐜 debug by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ) - prints all parameters passed to bot
- 📚 wikipedia by [@Macr0Nerd](https://github.com/Macr0Nerd) - returns a wikipedia article
- 🗿 translate by [@a-deeb](https://github.com/a-deeb) - google translate plugin
- 📑 test by [@Abdur-rahmaanJ](https://github.com/Abdur-rahmaanJ) - runs tests
- ⛅️ weather by [@Macr0Nerd](https://github.com/Macr0Nerd) - returns weather info for a given location
- ✉️ mail by [@TannerFry](https://github.com/TannerFry) - send emails within the chat
- 🕴️ hangman by [@JustinWalker4179](https://github.com/justinwalker4179) - play hangman in the chat
- 🎂 age by [@JustinWalker4179](https://github.com/justinwalker4179) - takes in birthday and outputs age
- ✔️ fact by [@JustinWalker4179](https://github.com/justinwalker4179) - returns a random fact
- 🔍 google by [@JustinWalker4179](https://github.com/justinwalker4179) - returns three search results from google
- 📮 send message by [@JustinWalker4179](https://github.com/justinwalker4179) - sends a message to another channel
- 📝 log by [@RiceAbove](https://github.com/RiceAbove) - logs the chat into a log file
- 🚪 joins by [@RiceAbove](https://github.com/RiceAbove) - greets everyone who joins the channel
- 📅 date by [@RiceAbove](https://github.com/RiceAbove) - posts the current date
- 🕵️‍ riddle by [@AngeloGiacco](https://github.com/AngeloGiacco) - returns a random riddle
- 🗞 news by [@AngeloGiacco](https://github.com/AngeloGiacco) - gets the top 10 headlines from bbc world news
- 📝 horoscope by [@AngeloGiacco](https://github.com/AngeloGiacco) - gets your daily horoscope for your starsign
- 💵 currency converter by [@AngeloGiacco](https://github.com/AngeloGiacco) - converts currencies
- 🔫 russian_roulette by [@AngeloGiacco](https://github.com/AngeloGiacco) - may or may not kick you off the channel
- 🏨 monopoly by [@AngeloGiacco](https://github.com/AngeloGiacco) - Honeybot now supports the world's worst game!
- ⚽️ transfer_rumour by [@AngeloGiacco](https://github.com/AngeloGiacco) - shows the day's transfer rumours
- 🃏 blackjack/21 by [@AngeloGiacco](https://github.com/AngeloGiacco) - play 21 on honeybot!
- 🤒 corona by [@AngeloGiacco](https://github.com/AngeloGiacco) - get the latest news on coronavirus with honeybot!
- 📖 diary by [@AngeloGiacco](https://github.com/AngeloGiacco) - keep a diary with honeybot
- 🛩 flight by [@AngeloGiacco](https://github.com/AngeloGiacco) - gets flight info from flightradar24
- 🎲 roll by [@GlennToms](https://github.com/GlennToms) - rolls a dice
- ❓ help by [@edumello](https://github.com/edumello) - show link to plugin's information page
- ✅ channeljoin by [@marceloyb](https://github.com/marceloyb) - join command for bot
- :page_with_curl: comic by [@mboekhold](https://github.com/mboekhold) - returns a random comic
- 📝 todo by [@h-ranjan1110](https://github.com/h-ranjan1110) - Makes a to do list .
- 🎱 Magic 8 Ball [@ZakariaTalhami](https://github.com/ZakariaTalhami) - Answer questions using magic 8 ball


## ⚡ Quickstart

setup

```bash
mkdir botx
cd botx
python -m venv venv
venv\scripts\activate # or source venv/bin/activate for linux
python -m pip install honeybot

```

run

```bash
honeybot init
honeybot run
```

in settings/CONNECT.conf
~~~
[INFO]

server_url = irc.libera.chat
port = 6667
name = appinventormuBot
~~~

In settings/PLUGINS.conf listed plugins under plugins/downloaded will load. Plugins listed under plugins/core are auto loaded.

## 💻 Seeing The Bot In Action

Get an IRC client

- Web: [Kiwiirc](https://kiwiirc.com) (easy)
- Desktop: [Hexchat](https://hexchat.github.io)
- Android: [Revolution IRC](https://github.com/MCMrARM/revolution-irc)

configure

```
port: 6667
url: chat.freenode.net
```

then join channel ```#ltch```

you should see the bot as hbot ... or as it's name is in [settings](https://github.com/pyhoneybot/honeybot/blob/master/honeybot/settings/CONNECT.conf)


## Docker
** Building Docker image **
Change SERVER, PORT and NICKNAME variables to match your preferences
```
docker build -t honeybot -f ./docker/Dockerfile .
```
** Running Docker image **
```
docker run honeybot
```
## 🔧 Plugins Development

Each plugin has a folder named after it. In it there is a file called main.py. In each there is a class called Plugin.

```python
# -*- coding: utf-8 -*-

class Plugin:
    def __init__(self):
        pass

    def run(self, incoming, methods, info, bot_info):
        try:
            if info['command'] == 'PRIVMSG' and info['args'][1] == '.hi':
                methods['send'](info['address'], 'hooo')
        except Exception as e:
            print('woops plugin error ', e)
```
we see four parameters being passed to the run method ```, incoming, methods, info, bot_info)```

#### parameter1: incoming

```incoming``` is the raw line and is not used except if you are not satisfied with the already provided methods

#### parameter2: methods

```methods``` is a dictionary of methods to ease your life. a quick look at [main.py](honeybot/main.py) reveals

```python
def methods(self):
        return {
                'send_raw': self.send,
                'send': self.send_target,
                'join': self.join
                }
```
where ```send_raw``` allows you to send in any string you want, thereby allowing you to implement any irc protocol from scratch

but, for most uses, ```send``` allows you to send a message to an address ```methods['send']('<address>', '<message>')```. using it in conjunction with info parameter allows you to send messages where it came from, in pm to the bot or in a channel. you can however hardcode the address.

```join``` allows you to join a channel by ```methods['join']('<channel name>')```

#### parameter3: info (meaning message info)

for a normal run, info produces
```python
{
'prefix': 'appinv!c5e342c5@gateway/web/cgi-irc/kiwiirc.com/ip.200.200.22.200',
'command': 'PRIVMSG',
'address': '##bottestingmu',
'args': ['##bottestingmu', 'ef']
}
```
hence if you want messages, ```messages = info['args'][1:]``` or the first word if you want to check for command will be ```info['args'][1]```

**use example**

- the command info is used in the join channel plugin to detect a join command and greet the user who just joined

### bot info

bot_info returns info about the bot

```python
        return {
            'name': self.name,
            'special_command': self.sp_command,
            'required_modules': self.required_modules,
            'owners': self.owners,
            'time': self.time,
            'friends': self.friends
        }
```

so that in run method you can access those.

**use example**

- For example, the time info is used in the uptime plugin by minussing it from the current time to get time bot has been running.
- The required modules is used in the installed plugin to determine what required plugin the bot runner did not install

#### wrapping up

hence
```python
if info['command'] == 'PRIVMSG' and info['args'][1] == '.hi':
    methods['send'](info['address'], 'hooo')
```
from above means
```
if message received == .hi:
    send(address, message)
```



## 📃 Contributing Guide

- don't forget to add your country flag here after accepted PR. i'll have to hunt it down on your profile if not.
- make sure to follow PEP8

**about PR**

first clone the project

```
git clone https://github.com/pyhoneybot/honeybot.git
```

cd into the project

```
cd honeybot
```

different changes to different files. for example, someone making a weather plugin first he creates a new branch
```
git checkout -b "weather-plugin"
```
then he commits
```
git add *
git commit -m "added weather plugin"
```

or

```
git commit -a -m "did this"
```

then he push to create a PR with the branch
```
git push origin head
```

or

```
git push origin weather-plugin
```

now let us say he wants to work on another issue, adding a joke in the jokes plugin, he creates another branch
```
git checkout -b "add-jokes"
```
after, same as before
```
git add *
git commit -m "added some jokes"
git push origin head
```
now he wants to fix his weather plugin, he changes branch
```
git checkout weather-plugin
```
works, then commit
```
git add *
git commit -m "fixed <issue>"
```
then a PR
```
git push origin head
```

**Why all these?**

So as not to reject a whole PR just because of some oddities. Reject only unneeded part.

**Updating the Documentation**

If you created a new plugin you should add your plugin to the documentation.
To do this, go into your cloned honeybot repo and then into the directory *docs/source/Plugins* .
Depending on the type of plugin write this into the development, fun, miscellaneous or utility RST file:

```rst
   
   <Plugin-Name>
   ^^^^^^^^^^^^^
   .. automodule:: plugins.<your-plugin-filename>
      :members:
```
	  
This allows sphinx to automatically pull the docstrings from the code of your plugin and parse them accordingly.

A small guide on how to further contribute to the documentation of the project can be found [here](https://pyhoneybot.github.io/honeybot/How_Tos/documentation.html)

## 🥄 Updating fork

Now, other changes are ongoing, what if you need the latest changes?
```
git pull origin master
```
helps if you cloned your own repo. What if you want to update your local copy of someone else's repo that you forked?
You do it like that

```
cd <your/local/cloned/repo/path/here>
git remote add upstream https://github.com/pyhoneybot/honeybot.git
git fetch upstream
git pull upstream master
```

## 🔌 Todo Plugins
- [x] 💐 humour
- [x] 🌨️ weather
- [x] ✉️ mail
- [x] 🎛️ maths
- [ ] 📥 pm when user online
- [ ] Random Colour

## ☑ Allowing Plugins
in PLUGINS.conf, add the plugin to allow on a new line !
~~~
calc
username
~~~

## 📧 Contact
### Email
- Abdur-Rahmaan Janhangeer | arj.python@gmail.com

## 🖊 Credits
[@arwinneil](https://github.com/arwinneil) for opensource and madeinmoris badges
